# caching_service
The caching service is the core service in the time-series cache system.  
With the cache you can improve availability of remote data and request it through a protobuf interface.  
Special consideration has been taken to ensuring a minimal usage of network connections, this way, data behind poor network connections can be requested with minimal impact on the underlying connection. 

## Contents
* [Getting started](#getting-started)
* [Environment variables](#environment-variables)
* [API](#api)

## Getting started
There are two options when running the cache service, running it with node, or with docker.

In either case, you also need to set up a [timescaledb](#timescale) database as well as an [mqtt broker](#mosquitto) to run it.  
Additionally, you need to set up vessels to be able to retrieve data. For testing, the `mockVessel` command can be used, or alternatively the [vesselmock docker image](https://hub.docker.com/repository/docker/davidrk/vesselmock) 

To run the entire time-series cache system, please check out the [docker repo](https://github.com/WireloggerProject/docker)

### Node
When using Node, you need node version 16.13 or newer.
You also need to create a [`.env`](./.env_template) file in the root directory, for more information about environment variables see [environment variables](#environment-variables).  
To start the cache first run `npm i`, then `npm run start`. Building the service is not necessary, as building is included in the start script.

### Docker
If you choose to use docker, you can either build this directory, or find the latest image at [davidrk/wlcache](https://hub.docker.com/repository/docker/davidrk/wlcache).
When using docker, environment variables must be loaded with the --env-file option passing in the path to a file following the [template](./.env_template).  
To run the broker with docker you can use the following command`docker run -d --env-file ./.env davidrk/wlcache`

## Environment variables
### MQTT
* MQTT_BROKER_ADDRESS: The ip/domain of the mqtt broker, for example `localhost`
* MQTT_USER: The username to log into the cache (optional)
* MQTT_PWD: The password to log into the cache (optional)
* MQTT_PORT: The port the mqtt broker runs on (Defaults to 1883 if TLS is disabled, 8883 if not)
* MQTT_REQUIRE_TLS: Set to false to allow non-TLS connections (defaults to true)

### Postgres
* DB_URL: The URL for the database, including port, for example `localhost:5432`
* DB_USER: The username to log into the database
* DB_PWD: The password to log into the database 
* DB_REQUIRE_TLS: Set to false to allow non-TLS connections (defaults to true)

## API
The cache service exposes an api for querying data and a simple command interface through MQTT  
`+`: Single-layer MQTT wildcard  
`#`: Multi-layer MQTT wildcard  
Every endpoint utilizes protobufs, each endpoint therefore specifies which protobuf it utilizes.
Since there are no statuscodes in MQTT, the protobufs include an optional error field that is filled whenever a request is unsuccessful

For more information on a given requests parameter(s), you can look at the proto definition

### Get range
Topic: `wlcache/data/+(vesselid)/request`  
Responsetopic: `wlcache/data/+(vesselid)/response`  
Proto: [Changes](./proto/cdplogreader.proto)  
Parameters:
* fromTo
* signal

Returns the data matching the given vessel, signals and fromto

### Discover
Topic : `wlcache/data/+(vesselid)/request`  
Responseopic : `wlcache/data/+(vesselid)/response`  
Proto: [Changes](./proto/cdplogreader.proto)  
Parameters:
* lastn (must be 0)

Returns the signals/variables available at the given vessel

### Get last n
Topic : `wlcache/data/+(vesselid)/request`  
Responseopic : `wlcache/data/+(vesselid)/response`  
Proto: [Changes](./proto/cdplogreader.proto)  
Parameters:
* lastn (cannot be 0)
* signal

Returns the last n readings for each of the given signals at the given vessel

### Get metadata
Topic : `wlcache/metadata/+(vesselid)/request`  
Responseopic : `wlcache/metadata/+(vesselid)/response`  
Proto: [MetaDataRequest and MetaDataResponse](./proto/wlHealthCheck.proto)  
Parameters:
* At least one variable
* from (optional)
* to (optional)

Returns a MetaDataResponse containing a list of metadata grouped by variable name

### Command
Topic : `wlcache/command/request`  
Responseopic : `wlcache/command/response/+(command)`  
Proto: [Command and CommandResponse](./proto/wlCommand.proto)  
Parameters:
* action
* vessel (optional)
* variable (optional)
* ranges (optional)

Performs a given action, or returns "Not implemented" if an action isn't implemented.  
Currently, only one action is implemented, which is "wipe"

#### Actions
* Wipe
  * Wipes data fitting within the given parameters:
    * Vessel: The vessel to wipe
    * Variable: The variables to wipe
    * Ranges: A range to wipe within (only data over from and under to)
      * Defaults to from 0 to infinity
      * If setting only from, all data after the from time will be deleted
      * If setting only to, all data before the to time will be deleted
      * If setting both, the data between will be deleted
  * If no parameters are given, the command will wipe the entire database

### Healthcheck
Topic : `wlcache/healthcheck/request`  
Responseopic : `wlcache/healthcheck/response`  
Proto: [HealthCheck](./proto/wlHealthCheck.proto)  
Sends a response-message with "ping" as the message every 5 seconds, no request is currently needed 

## Compiling proto files
To compile the proto files, use the following command:  
```protoc --plugin=./node_modules/.bin/protoc-gen-ts_proto.cmd --ts_proto_out=./src --ts_proto_opt=esModuleInterop=true --ts_proto_opt=forceLong=string ./<filename>.proto```

## Timescale
To quickly set up a timescaledb database for testing with docker, the following command can be used:  
``docker run -d -p 5432:5432 -e POSTGRES_PASSWORD=<PASSWORD> timescale/timescaledb:latest-pg14``

Another option is to use a [hosted database](https://www.timescale.com/products)
## Mosquitto
To quickly set up an eclipse mosquitto broker for testing use the following command:  
``docker run -d -p 1883:1883 -v <mosquitto.conf path>:/mosquitto/config/mosquitto.conf eclipse-mosquitto``  
Then, stop the container, go into the generated mosquitto.conf file and add the following to the end before restarting:  
```
listener 1883
allow_anonymous true
```