import faker from "@faker-js/faker";
import Reading, {ReadingBean} from "../models/reading";
import {Vessel} from "../models/vessel";
import {Variable} from "../models/variable";

/**
 * Generates test-data as specified by parameters
 * @param vesselName The vessel name to tie data to
 * @param names The names of the data to create
 * @param dataCount The amount of data to create for each name
 * @param timeSpread The the milliseconds from Date.now and back to create data for
 */
const generateData = async (vesselName: string|undefined|null,
                            names: number|string[],
                            dataCount: number,
                            timeSpread: number = 1e7): Promise<{
    records: Reading[],
    vesselName: string,
    dataNames: string[]
}> => {
    if (!vesselName) vesselName = faker.datatype.string(12)

    const dataNames: string[] = []
    if (typeof names == "number") {
        for (let i = 0; i < names; i++) {
            let n = faker.lorem.word()
            while(dataNames.includes(n)) n = faker.lorem.word()
            dataNames.push(n)
        }
    } else {
        dataNames.push(...names)
    }

    const dataValues: number[][] = []
    dataNames.forEach(dataname => {
        const dnArray: number[] = []
        for (let i = 0; i<dataCount; i++) {
            dnArray.push(faker.datatype.number())
        }
        dataValues.push(dnArray)
    })

    const vesselid = (await Vessel.findOrCreate({where: {name: vesselName}, logging: false}))[0].id

    const records: ReadingBean[] = []
    for (let i = 0; i < dataNames.length; i++) {
        const dn = dataNames[i]
        const signalId = (await Variable.findOrCreate({where: {name: dn}, logging: false}))[0].id
        for (let j = 0; j < dataValues[i].length; j++) {
            const record: ReadingBean = {
                vessel: vesselid,
                variable: signalId,
                value: dataValues[i][j],
                timestamp: Date.now()-Math.trunc(timeSpread*Math.random())
            }
            records.push(record)
        }
    }

    const savedData = await Reading.bulkCreate(records, {logging: false, ignoreDuplicates: true})
    return {
        records: <Reading[]>savedData,
        dataNames: dataNames,
        vesselName: vesselName
    };
}
export default generateData