import dotenv from 'dotenv'
import * as path from 'path'

/**
 * Loads configuration / environment variables from the .env file in root directory
 * If required parameters are missing it will notify through the terminal and close the cache
 */
const configEnv = () => {
  console.info('Loading environment variables...')
  dotenv.config({
    path: path.resolve(process.cwd(), '.env')
  })
  let misConfigured = false
  if (!process.env.DB_PWD) {
    misConfigured = true
    console.log('Missing required "DB_PWD" parameter')
  }
  if (!process.env.DB_URL) {
    misConfigured = true
    console.log('Missing required "DB_URL" parameter')
  }
  if (!process.env.DB_USER) {
    misConfigured = true
    console.log('Missing required "DB_USER" parameter')
  }
  if (!process.env.MQTT_BROKER_ADDRESS) {
    misConfigured = true
    console.log('Missing required "MQTT_BROKER_ADDRESS" parameter')
  }
  if (misConfigured) {
    console.error('Environment variables misconfigured, shutting down\n',
      'For more information, check readme')
    process.exit(1)
  }
  console.info('Environment variables loaded')
}

export default configEnv
