import { Command, CommandResponse } from '../../proto/wlCommand'
import wipeCache from './wipeCache'
import mqtt from 'mqtt'
import { COMMAND_RESPONSE_TOPIC } from '../../services/mqtt/topics'
import { NAMEVALIDATOR } from '../../models/vessel'

/**
 * Handles command requests and forwards it to the correct action handler
 *
 * @param {Command} command The decoded protobuf command
 * @param {mqtt.MqttClient} client The mqtt client to respond with
 */
const commandTopicHandler = async (command: Command, client: mqtt.MqttClient) => {
  if (command.vessel != null && !NAMEVALIDATOR.test(command.vessel)) {
    client.publish(COMMAND_RESPONSE_TOPIC(command.action),
      Buffer.from(CommandResponse.encode({ success: false, error: `Invalid vessel "${command.vessel}"` }).finish()))
    return
  }

  for (let i = 0; i < command.variables.length; i++) {
    const variable = command.variables[i]
    if (!NAMEVALIDATOR.test(variable)) {
      client.publish(COMMAND_RESPONSE_TOPIC(command.action),
        Buffer.from(CommandResponse.encode({ success: false, error: `Invalid variable "${variable}"` }).finish()))
      return
    }
  }

  try {
    let response: CommandResponse
    switch (command.action) {
      case 'wipe': {
        response = await wipeCache(command)
        break
      }
      default: {
        response = { success: false, error: 'Not implemented' }
      }
    }

    // Encode the response with protobuf and publish a response
    client.publish(COMMAND_RESPONSE_TOPIC(command.action), Buffer.from(CommandResponse.encode(response).finish()))
  } catch (e: any) {
    client.publish(COMMAND_RESPONSE_TOPIC(command.action), Buffer.from(CommandResponse.encode({ success: false, error: e.message }).finish()))
  }
}
export default commandTopicHandler
