import {
  Sequelize,
  INTEGER,
  Model,
  CreationOptional,
  InferAttributes,
  InferCreationAttributes, BIGINT
} from 'sequelize'
import { Variable } from './variable'
import { Vessel } from './vessel'

/**
 * Class representing a MetaData model
 * Extends the sequelize model definition, for more info, check out {@link https://sequelize.org/v6/manual/model-basics.html}
 */
export default class MetaData extends Model<InferAttributes<MetaData>, InferCreationAttributes<MetaData>> {
  declare id: CreationOptional<number>
  declare variable: number
  declare vessel: number
  declare from: number
  declare to: number
}

/**
 * Initializes the MetaData model and ties it to a sequelize instance
 *
 * @param {MetaData} sequelize the sequelize instance to instantiate with
 * @returns {MetaData} the metadata model
 */
export const initMetaData = (sequelize: Sequelize) => {
  return MetaData.init({
    id: {
      type: INTEGER,
      primaryKey: true,
      autoIncrement: true
    },
    variable: {
      type: INTEGER,
      references: {
        model: Variable,
        key: 'id'
      }
    },
    vessel: {
      type: INTEGER,
      references: {
        model: Vessel,
        key: 'id'
      }
    },
    from: {
      type: BIGINT
    },
    to: {
      type: BIGINT
    }
  }, {
    tableName: 'metadata',
    sequelize,
    timestamps: false
  })
}
