import { Vessel } from './vessel'
import {
  BIGINT,
  DOUBLE,
  InferAttributes,
  InferCreationAttributes,
  INTEGER,
  Model,
  Sequelize
} from 'sequelize'
import { Variable } from './variable'

/**
 * Utility interface for non-model representations of Reading
 */
export interface ReadingBean extends InferCreationAttributes<Reading> {
    variable: number;
    value: number;
    vessel: number;
    timestamp: number;
}

/**
 * Class representing a Reading model
 * Extends the sequelize model definition, for more info, check out {@link https://sequelize.org/v6/manual/model-basics.html}
 */
export default class Reading extends Model<InferAttributes<Reading>, InferCreationAttributes<Reading>> {
  declare variable: number
  declare value: number
  declare vessel: number
  declare timestamp: number
}

/**
 * Initializes the Reading model and ties it to a sequelize instance
 *
 * @param {Sequelize} sequelize the sequelize instance to instantiate with
 * @returns {Reading} The reading model
 */
export const initReading = (sequelize: Sequelize) => {
  return Reading.init({
    value: {
      type: DOUBLE,
      allowNull: false
    },

    timestamp: {
      type: BIGINT,
      primaryKey: true,
      allowNull: false
    },
    vessel: {
      type: INTEGER,
      references: {
        model: Vessel,
        key: 'id'
      },
      allowNull: false,
      primaryKey: true
    },
    variable: {
      type: INTEGER,
      references: {
        model: Variable,
        key: 'id'
      },
      allowNull: false,
      primaryKey: true
    }
  }, {
    tableName: 'reading',
    sequelize,
    timestamps: false
  })
}
