import {
  CreationOptional, HasManyCreateAssociationMixin, HasManyGetAssociationsMixin,
  InferAttributes,
  InferCreationAttributes,
  INTEGER,
  Model,
  NonAttribute,
  Sequelize, STRING
} from 'sequelize'
import Reading from './reading'

/**
 * Class representing a Variable model used to map readings and metadata to hashed variable names
 * Extends the sequelize model definition, for more info, check out {@link https://sequelize.org/v6/manual/model-basics.html}
 */
export class Variable extends Model<InferAttributes<Variable>, InferCreationAttributes<Variable>> {
  declare id: CreationOptional<number>
  declare name: string

  declare addReading: HasManyCreateAssociationMixin<Reading>
  declare getReadings: HasManyGetAssociationsMixin<Reading>

  declare readings: NonAttribute<Reading>
}

/**
 * Initializes the Variable model and ties it to a sequelize instance
 *
 * @param {Sequelize} sequelize the sequelize instance to instantiate with
 * @returns {Variable} The variable model
 */
export const initVariable = (sequelize: Sequelize) => {
  return Variable.init({
    id: {
      type: INTEGER,
      primaryKey: true,
      autoIncrement: true
    },
    name: {
      type: STRING(64),
      allowNull: false,
      unique: true
    }
  }, {
    tableName: 'variable',
    sequelize,
    timestamps: false
  })
}
