import {
  CreationOptional, HasManyCreateAssociationMixin, HasManyGetAssociationsMixin,
  InferAttributes,
  InferCreationAttributes,
  INTEGER,
  Model,
  NonAttribute,
  Sequelize, STRING
} from 'sequelize'
import Reading from './reading'

/**
 * Class representing a Vessel model used to map readings and metadata to hashed vessel names
 * Extends the sequelize model definition, for more info, check out {@link https://sequelize.org/v6/manual/model-basics.html}
 */
export class Vessel extends Model<InferAttributes<Vessel>, InferCreationAttributes<Vessel>> {
  declare id: CreationOptional<number>
  declare name: string

  declare addReading: HasManyCreateAssociationMixin<Reading>
  declare getReadings: HasManyGetAssociationsMixin<Reading>

  declare readings: NonAttribute<Reading>
}

/**
 * Initializes the Vessel model and ties it to a sequelize instance
 *
 * @param {Sequelize} sequelize the sequelize instance to instantiate with
 * @returns {Vessel} The vessel model
 */
export const initVessel = (sequelize: Sequelize) => {
  return Vessel.init({
    id: {
      type: INTEGER,
      primaryKey: true,
      autoIncrement: true
    },
    name: {
      type: STRING(64),
      allowNull: false,
      unique: true
    }
  }, {
    tableName: 'vessel',
    sequelize,
    timestamps: false
  })
}

/**
 * Regular expression to validate vessel- and variable-names
 */
export const NAMEVALIDATOR = /^[\w_-]{1,64}$/i
