/**
 * Interface for a ByteCounter
 * Used as a passable counter
 */
export interface ByteCounter {
    get: () => number;
    add: (a: number) => number
}

/**
 * Creates new bytecounter and returns it as function
 *
 * @returns {ByteCounter} A bytecounter instance
 */
const byteCounter = (): ByteCounter => {
  let count: number = 0

  return {
    /**
     * Retrieves the bytecounters accumulated value
     *
     * @returns {number} Accumulated number
     */
    get: () => count,

    /**
     * Increments the bytecounted
     *
     * @param {number} num The number to increment with
     * @returns {number} The new count
     */
    add: (num: number) => {
      count += num
      return count
    }
  }
}
export default byteCounter
