/**
 * Generic FIFO queue
 *
 * @template T
 */
export class Queue<T> {
  /**
   * Generic queue node
   *
   * @template T
   */
  static Node = class Node<T> {
    public value: T
    public next: Node<T>|undefined

    /**
     *
     * @param {T} value Creates a queue with the T generic
     */
    constructor (value: T) {
      this.value = value
    }
  }

  /**
   * Queue head
   *
   * @private
   */
  private head: typeof Queue.Node.prototype|undefined

  /**
   * Queue tail
   *
   * @private
   */
  private tail: typeof Queue.Node.prototype|undefined

  /**
   * Counter of how many items have been enqueued
   *
   * @private
   */
  private throughput: number = 0

  /**
   * Enqueues a given object of type T
   *
   * @param {T} t The object to enqueue
   */
  public enqueue (t: T): void {
    this.throughput++
    const newTail = new Queue.Node<T>(t)

    if (this.head == null) {
      this.head = newTail
    } else {
            this.tail!.next = newTail
    }
    this.tail = newTail
  }

  /**
   * Dequeues and returns the next object
   *
   * @returns {T} The next item in FIFO order
   * @throws {Error} If queue is empty
   */
  public dequeue (): T {
    if (this.head != null) {
      const headValue = this.head.value
      this.head = this.head.next
      return headValue
    } else {
      throw new Error('Queue is empty')
    }
  }

  /**
   * Returns true if the queue is not empty
   *
   * @returns {boolean} true if queue has more entries
   */
  public hasNext (): boolean {
    return this.head != null
  }
}
