import EntityDataController, { EntityData } from '../services/database/entityDataController'
import faker from '@faker-js/faker'
import fetchFromFile from '../handlers/data/getRange/fetchFromFile'
import { StreamStateEntry } from '../services/database/streamStateController'
import fs from 'fs'
import path from 'path'
import pg from 'pg'
import initModels from '../.jest/initModels'

jest.setTimeout(3e4)

beforeAll(async () => {
  pg.defaults.parseInt8 = true
  await initModels()
})

const datacount = 500
let veNode: EntityData
let vaNode: EntityData
let entry: StreamStateEntry
let from: number
let to: number

beforeEach(async () => {
  try {
    const veName = faker.name.lastName()
    veNode = await EntityDataController.vesselByName(veName)

    const vaName = faker.name.lastName()
    vaNode = await EntityDataController.variableByName(vaName)
  } catch (e) {
    console.log(e)
  }

  from = faker.datatype.number({ min: 1e5 })
  to = faker.datatype.number({ min: from + 1e5, max: from + 1e6 })
  const interval = (to - from) / datacount

  let tmpPath = path.resolve(process.cwd(), './tmp/test_' + Date.now() + '.csv')
  if (!fs.existsSync(path.resolve(process.cwd(), './tmp/'))) fs.mkdirSync(path.resolve(process.cwd(), './tmp/'))
  while (fs.existsSync(tmpPath)) tmpPath = path.resolve(process.cwd(), './tmp/test_' + Date.now() + '.csv')
  entry = new StreamStateEntry(veNode.name, vaNode.name, from, to)
  entry.setPath(tmpPath)

  const writeStream = fs.createWriteStream(tmpPath)

  for (let i = 0; i < datacount + 100; i++) {
    const timestamp = Math.trunc(from + ((i - 50) * interval))
    const value = faker.datatype.number({ precision: 0.01 })
    writeStream.write(`${value},${timestamp},${veNode.id},${vaNode.id}\n`)
  }

  writeStream.on('close', () => {
    entry.setStatus(true)
  })
  writeStream.close()

  await new Promise(resolve => {
    setTimeout(() => {
      resolve(null)
    }, 3e3)
  })

})

it('Should load the relevant data from the file once the entry status has changed', async () => {
  const res = await fetchFromFile(entry, from, to)

  // Some leeway for the sake of testing
  expect(Object.keys(res).length).toBeGreaterThanOrEqual(499)
  expect(Object.keys(res).length).toBeGreaterThanOrEqual(501)
  Object.keys(res).forEach(key => {
    expect(parseInt(key)).toBeGreaterThanOrEqual(from)
    expect(parseInt(key)).toBeLessThanOrEqual(to)
  })
})

it('Should propagate the error if the entry status changes to anything other than true', async () => {
  entry.setStatus(false)
  setTimeout(() => {
    entry.setStatus("Some error")
  }, 1e3)

  await expect(() => fetchFromFile(entry, from, to)).rejects.toThrowError("Entry status error")
})

it("Should throw error if the underlying file cannot be found", async () => {
  fs.unlinkSync(entry.localPath!)

  await expect(() => fetchFromFile(entry, from, to)).rejects.toThrowError("File does not exist")
})