import { InferCreationAttributes } from 'sequelize'
import Reading from '../models/reading'
import initModels from '../.jest/initModels'
import pg from 'pg'
import faker from '@faker-js/faker'
import EntityDataController, { EntityData } from '../services/database/entityDataController'
import loadRange from '../services/database/loadRange'
import { Signal } from '../proto/cdplogreader'

jest.setTimeout(1e4)

const namecount = 3
const datacount = 100

beforeAll(async () => {
  pg.defaults.parseInt8 = true
  await initModels()
})

let vesselNode: EntityData
let varNodes: EntityData[]

beforeEach(async () => {
  const names: string[] = []
  const dataByName: Record<string, number[]> = {}
  const timestampByName: Record<string, number[]> = {}
  for (let i = 0; i < namecount; i++) {
    let name = faker.name.lastName()
    while (names.includes(name)) name = faker.name.lastName()
    names.push(name)
    dataByName[name] = []
    timestampByName[name] = []
    for (let j = 0; j < datacount; j++) {
      dataByName[name].push(faker.datatype.number({ precision: 0.01 }))
      let ts = faker.datatype.number({ min: 0, max: 10000, precision: 1 })
      while (timestampByName[name].includes(ts)) ts = faker.datatype.number({ min: 0, max: 10000, precision: 1 })
      timestampByName[name].push(ts)
    }
  }

  const vesselname = faker.name.lastName()
  vesselNode = await EntityDataController.vesselByName(vesselname)

  varNodes = []
  for (let i = 0; i < names.length; i++) {
    const varNode = await EntityDataController.variableByName(names[i])
    varNodes.push(varNode)
  }

  const readings = names.reduce<InferCreationAttributes<Reading>[]>((acc, name, nidx) => {
    const arr = dataByName[name].map((value, idx) => {
      return {
        value,
        vessel: vesselNode.id,
        variable: varNodes[nidx].id,
        timestamp: timestampByName[name][idx]
      }
    })
    acc.push(...arr)
    return acc
  }, [])

  await Reading.bulkCreate(readings, { logging: false })
})

it('Should contain names*datacount entries', async () => {
  const sbn: Record<string, Signal> = {}
  await loadRange(vesselNode, varNodes, { from: 0, to: 10000 }, sbn)
  const names = Object.keys(sbn)
  expect(names.length).toBe(namecount)
  for (let i = 0; i < names.length; i++) {
    expect(Object.keys(sbn[names[i]].double).length).toBe(datacount)
  }
})
