import MetaData from '../models/metaData'
import { MetaDataAggregator } from '../services/database/metaDataAggregator'
import getMockMQTT from '../.jest/getMockMQTT'
import initModels from '../.jest/initModels'
import pg from 'pg'
import getMQTT from '../services/mqtt/getMQTTClient'
import dataStreamer from '../services/database/dataStreamer'
import { StreamStateEntry } from '../services/database/streamStateController'
import EntityDataController from '../services/database/entityDataController'
import MqttPool, { MqttPoolClient } from '../services/mqtt/mqttPool'

jest.setTimeout(1e4)

jest.mock('../services/mqtt/getMQTTClient')
const mqttMock = getMQTT as jest.MockedFunction<typeof getMQTT>
mqttMock.mockImplementation(getMockMQTT)

jest.mock('../services/mqtt/mqttPool', () => {
  const actual = jest.requireActual('../services/mqtt/mqttPool')
  actual.default.release = (client: MqttPoolClient) => client.end(true)
  return actual
})

jest.mock('../services/database/dataStreamer')
const sdMock = dataStreamer as jest.MockedFunction<typeof dataStreamer>
sdMock.mockImplementation(async (t: string, p: StreamStateEntry, os: () => void, oe: (e :any) => void) => {
  os()
})

beforeAll(async () => {
  pg.defaults.parseInt8 = true
  await initModels()
})

const range = {
  from: 1000,
  to: 2000
}

const vessel = 'Meta_Agg_VSL'
const variable = 'MDADATA'

it('Should aggregate metadata', async () => {
  const veNode = await EntityDataController.vesselByName(vessel)
  const vaNode = await EntityDataController.variableByName(variable)

  const m1 = new MetaData({
    vessel: veNode.id,
    variable: vaNode.id,
    from: range.from,
    to: range.from + Math.trunc((range.to - range.from) / 3)
  })

  const m2 = new MetaData({
    vessel: veNode.id,
    variable: vaNode.id,
    from: range.from + Math.trunc((range.to - range.from) / 3) + 10,
    to: range.to
  })

  const m3 = new MetaData({
    vessel: veNode.id,
    variable: vaNode.id,
    from: range.to + 2e4 + 1e3,
    to: m2.to + 1e5
  })

  const m4 = new MetaData({
    vessel: veNode.id,
    variable: vaNode.id,
    from: range.to + 1e3 - 1,
    to: range.to + 2e4
  })

  await m1.save({ logging: false })
  await m2.save({ logging: false })
  await m3.save({ logging: false })
  await m4.save({ logging: false })

  await expect(MetaData.findAll({ where: { vessel: veNode.id, variable: vaNode.id }, logging: false })
    .then(vals => vals.length)).resolves.toBe(4)

  // Metadata async again, must be re done
  MetaDataAggregator.get().add([m1, m2, m3, m4])
  await MetaDataAggregator.get().waitOnRun()

  // 4 intervals should be aggregated into one
  const dbMD = await MetaData.findAll({ where: { vessel: veNode.id, variable: vaNode.id }, logging: false })
  expect(dbMD.length).toBe(1)
  expect(dbMD[0].from).toBe(m1.from)
  expect(dbMD[0].to).toBe(m3.to)
})
