import { useColorModeValue, Box, Text, Divider } from '@chakra-ui/react'
import byteSize from 'byte-size'
import { ReactElement, useEffect, useState } from 'react'
import CountUp from 'react-countup'
import { useSiteState } from '../../utils/context/state'

/**
 * Functional Component for rendering current session usage data
 *
 * @returns {ReactElement} functional component
 */
const SessionUsage = (): ReactElement => {
  const [previous, setPrevious] = useState<any>({ bytecost: 0, bytetransfer: 0 })
  const { bytecost, bytetransfer } = useSiteState()

  useEffect(() => {
    setTimeout(() => {
      setPrevious({
        bytecost,
        bytetransfer
      })
    }, 200)
  }, [bytecost, bytetransfer])

  return (
    <Box pos={'absolute'} top={-1} zIndex={10} right={{ base: 10, xl: 0 }} w={150} bgColor={useColorModeValue('gray.100', 'gray.700')} rounded={'md'} shadow="md" py={1}>
      <Text fontSize={14} fontWeight={600} textAlign="center" px={2}>
        Session Usage
      </Text>
      <Divider my={2} />
      <CountUp start={previous.bytecost} end={bytecost} duration={3} separator=" " decimals={2} formattingFn={(value: number) => byteSize(value, { precision: 2 }) as any}>
        {({ countUpRef, start }) => (
          <Box display="flex" justifyContent="space-between" px={2}>
            <Text fontSize={12} fontWeight={600}>
              Cost
            </Text>
            <Text ref={countUpRef as any} fontSize={12} fontWeight={600}></Text>
          </Box>
        )}
      </CountUp>
      <CountUp start={previous.bytetransfer} end={bytetransfer} duration={3} separator=" " decimals={2} formattingFn={(value: number) => byteSize(value, { precision: 2 }) as any}>
        {({ countUpRef, start }) => (
          <Box display="flex" justifyContent="space-between" px={2}>
            <Text fontSize={12} fontWeight={600}>
              Transfer
            </Text>
            <Text ref={countUpRef as any} fontSize={12} fontWeight={600}></Text>
          </Box>
        )}
      </CountUp>
    </Box>
  )
}

export default SessionUsage
