import { FormControl, FormLabel, GridItem, Input } from '@chakra-ui/react'
import { ReactElement } from 'react'

interface Props {
  colSpan: number
  title: string
  accessor?: string
  value: any
  setValue: any
  type?: 'text' | 'number'
  placeholder?: string;
}

/**
 * Functional Component for rendering a GridInput, based on props it can handle getting and
 * setting of any value given as input.
 *
 * @param {Props} args Grid Input props
 * @returns {ReactElement<Props>} functional component
 */
const GridInput = ({ colSpan, title, accessor, value, setValue, type = 'text', placeholder = '' }: Props): ReactElement<Props> => {
  /**
   * Method for getting Input Value based on accessor or no accessor
   *
   * @param {any} value given input value
   * @returns {string} display value
   */
  const get = (value: any): string => {
    if (accessor) return value[accessor]
    return value
  }

  /**
   * Method for setting Input Value on change, it returns a function to be used by the
   * onChange callback
   *
   * @param {any} e event
   * @returns {void}
   */
  const set = (e: any): void => {
    if (accessor) return setValue({ ...value, [accessor]: e.target.value })
    return setValue(e.target.value)
  }

  return (
    <GridItem colSpan={colSpan}>
      <FormControl>
        <FormLabel>{title}</FormLabel>
        <Input placeholder={placeholder} variant="filled" type={type} value={get(value)} onChange={set} />
      </FormControl>
    </GridItem>
  )
}

export default GridInput
