import { ChevronDownIcon } from '@chakra-ui/icons'
import { Button, FormControl, FormLabel, GridItem, Menu, MenuButton, MenuList } from '@chakra-ui/react'
import { ReactElement } from 'react'
import { DateRange, DayPicker } from 'react-day-picker'
import { getFormattedDate } from '../../../utils/tools'

interface Props {
    range: DateRange | undefined
    setRange: any
    colSpan: number
    title: string
}

/**
 * Functional Component for Rendering Grid Range Picker, should be used to let user pick a data-range in a UI Grid
 *
 * @param {Props} args Grid Range Picker props
 * @returns {ReactElement<Props>} functional component
 */
const GridRangePicker = ({ range, setRange, colSpan, title }: Props): ReactElement<Props> => {
  /**
   * Method for getting Label of DatePicker
   *
   * @param {any} range unix range
   * @param {number} range.from unix from
   * @param {number} range.to unix from
   * @returns {string} formatted label
   */
  const getMenuLabel = (range: { from?: number; to?: number }): string => {
    if (!range) return 'Select Dates'
    if (range.from && range.to) {
      return 'From ' + getFormattedDate(range.from) + ' - To ' + getFormattedDate(range.to)
    }

    if (range.from) {
      return 'From ' + getFormattedDate(range.from)
    }
  }

  return (
    <GridItem colSpan={colSpan}>
      <FormControl>
        <FormLabel>{title}</FormLabel>
        <Menu>
          <MenuButton as={Button} flex="1" w="100%" rightIcon={<ChevronDownIcon />}>
            {getMenuLabel(range as any)}
          </MenuButton>
          <MenuList zIndex={1000}>
            <DayPicker
              mode="range"
              selected={range}
              onSelect={setRange}
              modifiersClassNames={{
                selected: 'selected',
                today: 'today'
              }}
            />
          </MenuList>
        </Menu>
      </FormControl>
    </GridItem>
  )
}

export default GridRangePicker
