import { Grid } from '@chakra-ui/react'
import { ReactElement, useEffect, useState } from 'react'
import { DateRange } from 'react-day-picker'
import { CommandMemory } from '../../../pages/commands'
import { format, parse } from '../../../utils/tools'
import GridInput from '../grid/gridInput'
import GridRangePicker from '../grid/gridRangePicker'

interface Props {
  memory: CommandMemory
  setMemory: any
}

/**
 * Functional Component for rendering Normal Panel component, used to select vessel, signal and data-range
 * to be used in the command line page
 *
 * @param {Props} args Normal Panel props
 * @returns {ReactElement<Props>} functional component
 */
const NormalPanel = ({ memory, setMemory }: Props): ReactElement<Props> => {
  const [time, setTime] = useState<string>('10:00')

  /**
   * Helper Method for handling change from time-input
   *
   * @param {string} value return value from change
   */
  const handleChange = (value: string): void => {
    let _time: number | string = parse(value.replace(':', ''))
    if (time.length >= 4) _time = ''
    setTime(_time as string)
  }

  useEffect(() => {
    if (memory.range) {
      const _time = time.toString().slice(0, 4)
      const hours = _time.slice(0, 2) || '0'
      const minutes = _time.slice(2, 4) || '0'

      let from: number = memory.range.from.setHours(parseInt(hours))
      from = new Date(from).setMinutes(parseInt(minutes))
      let to: number = memory.range.to.setHours(parseInt(hours))
      to = new Date(to).setMinutes(parseInt(minutes))

      console.log(from, to)
    }
  }, [time])

  return (
    <Grid gap="3" templateColumns="repeat(2, 1fr)">
      <GridInput colSpan={2} title="Vessel Name" accessor="vesselid" value={memory} setValue={setMemory} />
      <GridInput colSpan={2} title="Signals" accessor="signals" value={memory} setValue={setMemory} />
      <GridRangePicker colSpan={2} title="Range" range={memory.range} setRange={(range: DateRange) => setMemory({ ...memory, range })} />
      <GridInput colSpan={2} placeholder="10:00" title="Time" value={format(time)} setValue={handleChange} />
    </Grid>
  )
}

export default NormalPanel
