import { Button, GridItem, Radio, RadioGroup, Stack } from '@chakra-ui/react'
import { ReactElement } from 'react'

interface Props {
  onSubmit: Function
  type: string
  setType: any
  isLoading: boolean;
}

/**
 * Functional Component for rendering action part of Commands page,
 * consists of RadioButtons to selects type of command "metadata" | "range" | "wipe"
 * as well as Submit Button
 *
 * @param {Props} args Command Submit props
 * @returns {ReactElement<Props>} functional component
 */
const CommandsSubmit = ({ onSubmit, setType, type, isLoading }: Props): ReactElement<Props> => {
  return (
    <GridItem colSpan={2} mt={6}>
      <RadioGroup onChange={setType} value={type}>
        <Stack direction="row">
          <Radio colorScheme="green" value="metadata">
            Metadata
          </Radio>
          <Radio colorScheme="green" value="range">
            Range
          </Radio>
          <Radio colorScheme="green" value="wipe">
            Wipe
          </Radio>
        </Stack>
      </RadioGroup>
      <Button isLoading={isLoading} width="100%" onClick={onSubmit as any} mt={3}>
        Send
      </Button>
    </GridItem>
  )
}

export default CommandsSubmit
