import { GridItem, Tab, TabList, TabPanels, Tabs } from '@chakra-ui/react'
import { ReactElement } from 'react'

interface Props {
  tabs: string[]
  children: any
}

/**
 * Functional Component for wrapping Tab Panels for Command Page
 *
 * @param {Props} args Tabs props
 * @returns {ReactElement<Props>} functional component
 */
const CommandsTabs = ({ tabs, children }: Props): ReactElement<Props> => {
  return (
    <GridItem colSpan={2}>
      <Tabs isFitted colorScheme="gray">
        <TabList>
          {tabs.map((tab: string, idx: number) => (
            <Tab key={idx}>{tab}</Tab>
          ))}
        </TabList>
        <TabPanels>{children}</TabPanels>
      </Tabs>
    </GridItem>
  )
}

export default CommandsTabs
