import { Box, Flex, Grid, Text, useColorMode } from '@chakra-ui/react'
import dynamic from 'next/dynamic'
import { ReactElement, ReactNode } from 'react'
import { HiOutlineTerminal } from 'react-icons/hi'
import HeadComponent from '../shared/head'

// Need dynamic import as ReactJson needs access to "document" object
const ReactJson = dynamic(() => import('react-json-view'), { ssr: false })

interface Props {
  response: any
  children: ReactNode
}

/**
 * Functional Component for Wrapping entire Commands Page, includes head component
 *
 * @param {Props} args Wrapper props
 * @returns {ReactElement<Props>} functional component
 */
const CommandsWrapper = ({ response, children }: Props): ReactElement<Props> => {
  const { colorMode } = useColorMode()

  return (
    <Box display="flex" flexDir="column" flex="1" maxW={'6xl'}>
      <HeadComponent title="Commands" />
      <Flex alignItems="center" mx="auto">
        <Text fontSize="3xl" mr={3}>
          Command Interface
        </Text>
        <HiOutlineTerminal size={35} />
      </Flex>
      <Flex flexDir={{ base: 'column', md: 'row' }} gap={6} pl={{ base: 6, xl: 0 }}>
        <Grid h={{ base: '100%', md: '0%' }} w={{ base: '100%', md: '50%' }} >{children}</Grid>
        <Box flex="2" mt={1} w={{ base: '100%', md: '50%' }}>
          <Text mb={3} fontWeight={500}>
            Response
          </Text>
          <ReactJson src={response} name={false} displayObjectSize={false} displayDataTypes={false} enableClipboard={false} theme={colorMode === 'dark' ? 'ashes' : 'bright:inverted'} />
        </Box>
      </Flex>
    </Box>
  )
}

export default CommandsWrapper
