import { Box, Stack, useColorModeValue } from '@chakra-ui/react'
import { ReactElement, ReactNode } from 'react'

interface Props {
  children: ReactNode
  isOpen: boolean
}

/**
 * Functional Component for rendering mobile version of nav bar
 *
 * @param {Props} args mobile nav props
 * @returns {ReactElement<Props>} functional component
 */
const MobileNav = ({ children, isOpen }: Props): ReactElement<Props> => {
  return (
    <>
      {isOpen && (
        <Box
          pos="absolute"
          zIndex={2}
          left={0}
          right={0}
          pb={4}
          px={4}
          display={{ md: 'none' }}
          bgGradient={useColorModeValue('linear(to-l, gray.50, gray.200)', 'linear(to-l, gray.800, gray.900)')}
        >
          <Box py={4}>
            <Box h={16}></Box>
          </Box>
          <Stack as={'nav'} spacing={3}>
            {children}
          </Stack>
        </Box>
      )}
    </>
  )
}

export default MobileNav
