import { Container, Stack, Text, useColorMode, Flex, Image } from '@chakra-ui/react'
import { ReactElement } from 'react'

/**
 * Functional Component for rendering application footer
 *
 * @returns {ReactElement} functional component
 */
const Footer = (): ReactElement => {
  const { colorMode } = useColorMode()
  return (
    <Container
      zIndex={1}
      variant="base"
      as={Stack}
      maxW={'8xl'}
      py={4}
      direction={{ base: 'column', md: 'row' }}
      spacing={4}
      justify={{ base: 'center', md: 'space-between' }}
      align={{ base: 'center', md: 'center' }}
    >
      <Flex justify="space-between" alignItems="center" wrap="wrap" w="100%">
        <Text opacity="0.4" fontWeight="semibold" mb={{ base: '2rem', sm: '0' }}>
          © 2022 NTNU IIR. All rights reserved
        </Text>
        <a target="_blank" href="https://www.ntnu.no/">
          <Image src="/images/ntnu.png" opacity="0.4" filter={colorMode === 'light' ? 'invert(15%)' : 'invert(55%)'} alt="NTNU" height={50} />
        </a>
        <a target="_blank" href="https://www.seaonics.com/">
          <Image src="/images/seaonics.png" opacity="0.4" filter={colorMode === 'light' ? 'invert(15%)' : 'invert(55%)'} alt="Seaonics" height={50} />
        </a>
      </Flex>
    </Container>
  )
}

export default Footer
