import Head from 'next/head'
import { ReactElement, ReactNode } from 'react'
import packageJson from '../../package.json'
import { normalizeName } from '../../utils/tools'

interface Props {
  title: string
  children?: ReactNode | undefined
}

/**
 * Functional Component For normalizing <head> tag of all pages
 *
 * @param {Props} args head props
 * @returns {ReactElement<Props>} functional component
 */
const HeadComponent = ({ title, children }: Props): ReactElement<Props> => {
  return (
    <Head>
      <title>
        {title}
        {' | '}
        {normalizeName(packageJson.name)}
      </title>
      {children && children}
    </Head>
  )
}

export default HeadComponent
