import { Container, Flex, Stack } from '@chakra-ui/react'
import { ReactElement, ReactNode } from 'react'
import Footer from './footer'
import NavBar from '../navbar/navbar'
import SideBar from './sidebar'

interface Props {
  children: ReactNode
  pageProps: any
}

/**
 *  Functional Component for rendering Application Layout Wrapper
 *
 * @param {Props} args layout props
 * @returns {ReactElement<Props>} functional component
 */
const Layout = ({ children, pageProps }: Props): ReactElement<Props> => {
  return (
    <Flex minH="100vh" direction="column">
      <NavBar />
      <Container
        variant="base"
        padding={{ base: 3, md: 0 }}
        flex="1"
        as={Stack}
        mx="auto"
        maxW={'6xl'}
        direction={{ base: 'row', md: 'row' }}
        justify={{ base: 'start', md: 'start' }}
        align={{ base: 'start', md: 'start' }}
        pr={{ base: 6, xl: 0 }}
      >
        <SideBar>{children}</SideBar>
      </Container>
      <Footer />
    </Flex>
  )
}

export default Layout
