import { Box, useColorModeValue } from '@chakra-ui/react'
import Link from 'next/link'
import { ReactElement, ReactNode } from 'react'

interface Props {
  children: ReactNode
  href: string
  active?: boolean
  activeColor?: string
}

/**
 * Functional Component for rendering NavLinks
 *
 * @param {Props} args nav link props
 * @returns {ReactElement<Props>} functional component
 */
const NavLink = ({ children, href, active = false, activeColor }: Props): ReactElement<Props> => (
  <Link href={href}>
    <Box
      flex={1}
      px={2}
      py={1}
      cursor="pointer"
      outline="0 0 0 3px var(--chakra-colors-gray-500)"
      rounded={'md'}
      opacity={active ? '1' : '0.7'}
      bgColor={active ? (!activeColor ? useColorModeValue('gray.50', 'gray.800') : activeColor) : ''}
      shadow={active ? useColorModeValue('', 'md') : ''}
      fontWeight={500}
      _hover={{
        bg: useColorModeValue('gray.300', 'gray.700')
      }}
      display="flex"
      alignItems="center"
      justifyContent="space-between"
      isTruncated
    >
      {children}
    </Box>
  </Link>
)

export default NavLink
