import { Button, Text, ModalHeader, Input, Modal, ModalBody, ModalContent, ModalFooter, ModalOverlay } from '@chakra-ui/react'
import { ReactElement, useEffect, useState } from 'react'
import { useSiteState } from '../../utils/context/state'
import ToastHandler from '../../utils/handlers/toastHandler'
import { NAMEVALIDATOR } from '../../utils/tools'
/**
 * Functional Component for rendering Vessel Add Modal
 *
 * @returns {ReactElement} functional component
 */
const VesselModal = (): ReactElement => {
  const [name, setName] = useState('')
  const [error, setError] = useState<null | string>(null)
  const { isOpen, set } = useSiteState()
  const { successToast } = ToastHandler()

  useEffect(() => {
    if (isOpen) {
      setName('')
      setError(null)
    }
  }, [isOpen])

  /**
   * Helper Method for adding a new Vessel to the page
   */
  const handleAdd = () => {
    const _name = name.replaceAll(' ', '')
    if (!NAMEVALIDATOR.test(_name)) {
      setError('Invalid vessel name')
      return
    }
    set._vessels((v: string[]) => [...v, _name])
    successToast('Added Successfully', `New vessel '${name}' was added!`)
    set.onClose()
  }

  return (
    <Modal isOpen={isOpen} onClose={set.onClose} isCentered>
      <ModalOverlay />
      <ModalContent>
        <ModalHeader textAlign="center">Add Vessel</ModalHeader>
        <ModalBody w="100%" pb={5}>
          <Input name="search" type="text" variant="filled" placeholder="Name" onChange={(e: any) => setName(e.target.value)} />
          <Text py={2} color="red.400">{error}</Text>
        </ModalBody>
        <ModalFooter>
          <Button colorScheme="blue" mr={3} onClick={set.onClose}>
            Close
          </Button>
          <Button variant="ghost" onClick={handleAdd} disabled={name.replaceAll(' ', '') === ''}>
            Add
          </Button>
        </ModalFooter>
      </ModalContent>
    </Modal>
  )
}

export default VesselModal
