import { useDisclosure } from '@chakra-ui/react'
import React, { createContext, ReactElement, ReactNode, useContext, useRef, useState } from 'react'
import { Signal } from '../../globals'

/**
 * Anonymous function for default function value
 */
const anonymous = () => {}

const contextDefaultValues: any = {
  signalnames: [],
  signals: [],
  isLoading: false,
  isOpen: false,
  lastN: 200,
  chartRef: { current: null } as any,
  expanded: false,

  set: {
    _signalnames: anonymous,
    _signals: anonymous,
    _isLoading: anonymous,
    onOpen: anonymous,
    onClose: anonymous,
    _lastN: anonymous,
    _expanded: anonymous
  }
}

const Context = createContext<any>(contextDefaultValues)

/**
 * Accessor for all Signal Context variables and methods
 *
 * @returns {any} variables and methods
 */
export const useSignalContext = (): any => {
  return useContext(Context)
}

type Props = {
  children: ReactNode
}

/**
 * Wrapper Component for providing children with context
 *
 * @param {Props} args provider arguments
 * @returns {ReactElement<Props>} function wrapper component
 */
export const SignalProvider = ({ children }: Props): ReactElement<Props> => {
  const [signalnames, _signalnames] = useState<string[]>([])
  const [signals, _signals] = useState<Signal[]>([])
  const [lastN, _lastN] = useState<number>(200)
  const [isLoading, _isLoading] = useState<boolean>(false)
  const { isOpen, onOpen, onClose } = useDisclosure()
  const chartRef = useRef<any>(null)
  const [expanded, _expanded] = useState<boolean>(false)

  const value = {
    signalnames,
    signals,
    isLoading,
    isOpen,
    lastN,
    chartRef,
    expanded,

    set: {
      _signalnames,
      _signals,
      _isLoading,
      onOpen,
      onClose,
      _lastN,
      _expanded
    }
  }

  return <Context.Provider value={value}>{children}</Context.Provider>
}
