import { SearchIcon } from '@chakra-ui/icons'
import { Box, Input, InputGroup, InputLeftElement, Modal, Text, ModalBody, ModalContent, ModalOverlay } from '@chakra-ui/react'
import { ReactElement, useEffect, useState } from 'react'
import Items from '../shared/items'
import Pagenation from '../shared/pagenation'
import { useSignalContext } from './context'

interface Props {
  handleAdd: Function
}

/**
 * Functional Component for rendering Signal Picker Modal
 *
 * @param {Props} args signal modal props
 * @returns {ReactElement} functional component
 */
const SignalModal = ({ handleAdd }: Props): ReactElement<Props> => {
  const [items, setItems] = useState<any[] | null>(null)
  const [filtered, setFiltered] = useState<any[]>([])
  const [search, setSearch] = useState('')
  const { signalnames, set, isOpen } = useSignalContext()

  useEffect(() => {
    if (signalnames) setFiltered(signalnames)
  }, [signalnames])

  useEffect(() => {
    if (!isOpen) handleSearch('')
  }, [isOpen])

  /**
   * Method for handling searching of signals
   *
   * @param {string} key search key
   */
  const handleSearch = (key: string): void => {
    setSearch(key)
    if (!items) return
    setFiltered(signalnames.filter((signal: string) => signal.toLowerCase().includes(key.toLowerCase())))
  }

  return (
    <Modal isOpen={isOpen} onClose={set.onClose}>
      <ModalOverlay />
      <ModalContent>
        {signalnames && (
          <ModalBody w="100%" pb={5}>
            <InputGroup mx="auto" mt={5}>
              <InputLeftElement pointerEvents="none" children={<SearchIcon color="gray.300" />} />
              <Input name="search" type="text" variant="filled" placeholder="Search" onChange={(e: any) => handleSearch(e.target.value)} />
            </InputGroup>
            <Text mt={2} mb={4} ml={2} opacity={search === '' ? 0 : 0.7}>
              Found {filtered.length} matches for "{search}"
            </Text>
            {signalnames.length > 0 && (
              <Pagenation items={filtered} setItems={setItems} page={0} totalLength={signalnames.length}>
                <Box flexGrow="1" flex="1" h="30rem" mb={5}>
                  <Items onClick={handleAdd} small={true} currentItems={items || []} />
                </Box>
              </Pagenation>
            )}
          </ModalBody>
        )}
      </ModalContent>
    </Modal>
  )
}

export default SignalModal
