import { DataPoint, Indexable } from 'downsample'

export interface NetworkResponse {
  data: any;
  status: number;
}

export interface HandledResponse {
  data: any;
  message?: string;
}

export interface SuccessResponse {
  bytecost?: number;
  bytetransfer?: number;
  signal?: Signal[];
  signals?: string[];
}

export interface RequestPayload {
  next: Function;
  error: Function;
}

export interface FetchResponse {
  signal: Signal[];
  bytetransfer: number;
  byteconst: number;
}

export interface DiscoverResponse {
  signals: string[];
  bytetransfer: number;
  byteconst: number;
}

export interface RouterQuery {
  vesselid: string;
  page: string;
  signalid: string;
  compare: string;
  from: string;
  to: string
}

export interface Signal {
  name: string;
  double: {
    [key: string]: number;
  };
}

export interface Dataset {
  label: string;
  type: 'line' | 'bar' | 'doughnut';
  borderWidth: number;
  yAxisID: string;
  radius: number;
  tension: number;
  fill: boolean;
  data: Indexable<DataPoint>;
  stepped: any;
  parsing?: boolean;
  indexAxis?: 'x' | 'y';
  spanGaps: boolean;
  backgroundColor: string;
  borderColor: string
}

/** PAYLOAD TYPES */

export interface Payload {
  next: { (data: SuccessResponse) };
  error: { (data: HandledResponse) };
}

export interface VesselPayload extends Payload {
  vesselid: string;
}

export interface DiscoverPayload extends VesselPayload {}

export interface LastNPayload extends VesselPayload {
  signals: string[];
  lastn: number;
}

export interface RangePayload extends VesselPayload {
  signals: string[];
  from: number;
  to: number;
}
