import { ChakraProvider } from '@chakra-ui/react'
import Layout from '../components/shared/layout'
import theme from '../styles/theme'
import { StateProvider } from '../utils/context/state'
import 'react-day-picker/dist/style.css'
import '../styles/styles.css'
import { SignalProvider } from '../components/signal/context'
import { ReactElement } from 'react'

interface Props {
  Component: any
  pageProps: any
}

/**
 * Functional Component that acts as entry point and wrapper for application
 *
 * @param {Props} args App props
 * @returns {ReactElement<Props>} functional component
 */
const App = ({ Component, pageProps }: Props): ReactElement<Props> => {
  return (
    <StateProvider>
      <ChakraProvider theme={theme} resetCSS>
        <SignalProvider>
          <Layout pageProps={pageProps}>
            <Component {...pageProps} />
          </Layout>
        </SignalProvider>
      </ChakraProvider>
    </StateProvider>
  )
}

export default App
