import { TabPanel } from '@chakra-ui/react'
import { ReactElement, useState } from 'react'
import { DateRange } from 'react-day-picker'
import NormalPanel from '../../components/commands/panels/normal'
import UnixPanel from '../../components/commands/panels/unix'
import CommandsSubmit from '../../components/commands/submit'
import CommandsTabs from '../../components/commands/tabs'
import CommandsWrapper from '../../components/commands/wrapper'
import { wipe } from '../../services/commands'
import { getMetadata } from '../../services/metadata'
import { getRange } from '../../services/vessel'

export interface CommandMemory {
  vesselid: string
  signals: string
  range: DateRange | undefined
}

/**
 * Functional Component for rendering Commands Page at /commands
 *
 * @returns {ReactElement} functional component
 */
const Commands = (): ReactElement => {
  const [response, setResponse] = useState<any>({})
  const [isLoading, setIsLoading] = useState<boolean>(false)
  const [type, setType] = useState('metadata')
  const [memory, setMemory] = useState<CommandMemory>({ vesselid: '', signals: '', range: undefined })

  /**
   * Helper Method for getting correct range format when submitting request
   *
   * @returns {DateRange} from, to
   */
  const handleRange = (): { from: number; to: number } => {
    if (memory.range) return { from: memory.range.from.getTime(), to: memory.range.to.getTime() }
    return { from: 0, to: 0 }
  }

  /**
   * Method for sending Command request
   */
  const onSubmit = async () => {
    setIsLoading(true)
    const payload = { vesselid: memory.vesselid, signals: memory.signals.replaceAll(' ', '').split(','), ...handleRange() }
    let res: any = {}
    if (type === 'range') res = await getRange(payload)
    if (type === 'wipe') res = await wipe(payload)
    if (type === 'metadata') res = await getMetadata(payload)
    setIsLoading(false)
    setResponse(res)
  }

  return (
      <CommandsWrapper response={{ response: response || { message: 'Something went wrong' } }}>
      <CommandsTabs tabs={['Unix Timestamps', 'Regular']}>
        <TabPanel p={0} pt={3}>
          <UnixPanel {...{ memory, setMemory }} />
        </TabPanel>
        <TabPanel p={0} pt={3}>
          <NormalPanel {...{ memory, setMemory }} />
        </TabPanel>
      </CommandsTabs>
      <CommandsSubmit {...{ onSubmit, type, setType, isLoading }} />
    </CommandsWrapper>
  )
}

export default Commands
