import axios, { AxiosError, AxiosResponse } from 'axios'
import { NetworkResponse } from '../globals'
import { handleErrorResponse, handleResponse } from './responseHandlers'
import getConfig from 'next/config'
// Get ENV at runtime
const { publicRuntimeConfig } = getConfig()

/**
 * AbortController instance connected to all call methods
 */
let controller = new AbortController()

/**
 * Instance of axios connected to api
 */
const CACHE_API = axios.create({
  baseURL: publicRuntimeConfig.cache_url,
  withCredentials: false
})

/**
 * Method for calling GET to the api
 *
 * @param {string} url endpoint string
 * @returns {Promise<object>} data
 */
export const _get = (url: string): Promise<NetworkResponse | string> => {
  return CACHE_API
    .get(url, {
      signal: controller.signal
    })
    .then((response: AxiosResponse<any, any>) => handleResponse(response))
    .catch((error: AxiosError<any, any>) => handleErrorResponse(error))
}

/**
 * Method for calling POST to the api
 *
 * @param {string} url endpoint string
 * @param {object} body post body
 * @returns {Promise<object>} data
 */
export const _post = (url: string, body: object): Promise<NetworkResponse | string> => {
  return CACHE_API
    .post(url, body, {
      signal: controller.signal
    })
    .then((response: AxiosResponse<any, any>) => handleResponse(response))
    .catch((error: AxiosError<any, any>) => handleErrorResponse(error))
}

// TODO: implement PUT and DELETE if needed

/**
 * Method to Cancel all current outgoing requests
 * Can be re-used from anywhere withing the project
 */
export const cancelAll = (): void => {
  controller.abort()
  controller = new AbortController()
}
