import { AxiosError, AxiosResponse } from 'axios'
import { NetworkResponse } from '../globals'

/**
 * Method for handing axios catch()
 *
 * @param {AxiosError<any, any>} error Axios error
 * @returns {NetworkResponse} response
 */
export const handleErrorResponse = (error: AxiosError<any, any>): NetworkResponse | string => {
  if (error.response) {
    // Request made and server responded
    return { data: error.response.data, status: error.response.status }
  } else if (error.request) {
    // The request was made but no response was received
    return error.request
  } else {
    // Something happened in setting up the request that triggered an Error
    return error.message
  }
}

/**
 * Method for handling axios then()
 *
 * @param {AxiosResponse<any, any>} response Axios response
 * @returns {NetworkResponse} response
 */
export const handleResponse = (response: AxiosResponse<any, any>): NetworkResponse => {
  return {
    data: response.data,
    status: response.status
  }
}
