import { LastNPayload, NetworkResponse, RangePayload, VesselPayload } from '../globals'
import { _get } from './methods'

/**
 * Method for calling raw discover call
 *
 * @param {VesselPayload} args data arguments
 * @returns {Promise<NetworkResponse | string>} Promise
 */
export const discover = ({ vesselid }: Partial<VesselPayload>): Promise<NetworkResponse | string> => _get(`${vesselid}/discover`)

/**
 * Method for calling raw lastN (multiple) call
 *
 * @param {LastNPayload} args data arguments
 * @returns {Promise<NetworkResponse | string>} Promise
 */
export const getLastN = ({ vesselid, signals, lastn }: Partial<LastNPayload>): Promise<NetworkResponse | string> => _get(`${vesselid}/data/${signals.toString()}?lastn=${lastn}`)

/**
 * Method for calling raw range (multiple) call
 *
 * @param {RangePayload} args data arguments
 * @returns {Promise<NetworkResponse | string>} Promise
 */
export const getRange = ({ vesselid, signals, from, to }: Partial<RangePayload>): Promise<NetworkResponse | string> => _get(`${vesselid}/data/${signals.toString()}?from=${from}&to=${to}`)
