import { useDisclosure } from '@chakra-ui/react'
import React, { createContext, ReactElement, ReactNode, useContext, useState } from 'react'
import { useLocalStorage } from '../hooks/useLocalstorage'
/**
 * Anonymous function for default function value
 */
const anonymous = () => {}

const contextDefaultValues: any = {
  bytecost: 0,
  bytetransfer: 0,
  vessels: [],

  set: {
    _bytecost: anonymous,
    _bytetransfer: anonymous,
    _vessels: anonymous
  }
}

const Context = createContext<any>(contextDefaultValues)

/**
 * Accessor for all Global Context variables and methods
 *
 * @returns {any} variables and methods
 */
export const useSiteState = () => {
  return useContext(Context)
}

type Props = {
  children: ReactNode
}

/**
 * Wrapper Component for providing children with context
 *
 * @param {Props} args provider arguments
 * @returns {ReactElement<Props>} function wrapper component
 */
export const StateProvider = ({ children }: Props): ReactElement<Props> => {
  const [vessels, _vessels] = useLocalStorage<string[]>('wirelogger.vessels', [])
  const [bytecost, _bytecost] = useState<number>(0)
  const [bytetransfer, _bytetransfer] = useState<number>(0)
  const { isOpen, onOpen, onClose } = useDisclosure()

  const value = {
    bytecost,
    bytetransfer,
    vessels,
    isOpen,

    set: {
      _bytecost,
      _bytetransfer,
      _vessels,
      onOpen,
      onClose
    }
  }

  return <Context.Provider value={value}>{children}</Context.Provider>
}
