# rest_service
The rest service is an HTTP proxy plugin for the [time series cache service](https://github.com/WireloggerProject/caching_service)  
With it, the cache service can be extended for web-usage.

## Contents
* [Getting started](#getting-started)
* [Environment variables](#environment-variables)
* [API](#api)

## Getting started
There are two options when running the rest service, running it with node, or with docker.  
Running the rest service also requires an MQTT broker with a running [cache service](https://github.com/WireloggerProject/caching_service) instance.  
For instructions to set up both the MQTT broker and the cache service, please look at the cache service readme.  

To run the entire time-series cache system, please check out the [docker repo](https://github.com/WireloggerProject/docker)
### Node
When using Node, you need node version 16.13 or newer.
You also need to create a .env file in the root directory, for more information about environment variables see [environment variables](#environment-variables).  
To start the cache first run `npm i`, then `npm run start`. Building the service is not necessary, as building is included in the start script.

### Docker
If you choose to use docker, you can either build this directory, or find the latest image at [davidrk/wlrest](https://hub.docker.com/repository/docker/davidrk/wlrest).
When using docker, environment variables must be loaded with the --env-file option.  
To run the broker with docker you can use the following command:  
`docker run -d -p 3000:3000 --env-file ./.env davidrk/wlrest`


## Environment variables
* MQTT_BROKER_ADDRESS: The ip/domain of the mqtt broker, for example `localhost`
* MQTT_USER: The username to log into the cache (optional)
* MQTT_PWD: The password to log into the cache (optional)
* MQTT_PORT: The port the mqtt broker runs on (Defaults to 1883 if TLS is disabled, 8883 if not)
* MQTT_REQUIRE_TLS: Set to false to allow non-TLS connections (defaults to true)
* PORT: The port on which the service should listen (defaults to 3000)

## API
Paths are given as they are set in expressjs, with `:path` meaning a path parameter

### Discover available signals for a vessel
Path: `/:vessel/discover`  
Method: GET  
Parameters: 
* vessel (required): The id of the vessel you want to request

Returns an array of all available signals on the specified vessel

### Get data in range
Path: `/:vessel/data/:data`  
Method: GET  
Parameters:
* vessel (required): The id of the vessel you want to request
* data (required): A comma seperated list of signals/variables to include in the request  

Query:
* from (required): The unix timestamp that is the start of the request range
* to (required): The unix timestamp that is the end of the request range

Returns the data from the vessel in the given range

### Get the last n data signals
Path: `/:vessel/data/:data`  
Method: GET  
Parameters:  
* vessel (required): The id of the vessel you want to request
* data (required): A comma seperated list of signals/variables to include in the request

Query:
* lastn (required): How many datapoints to request

Returns the last n datapoints from the vessel

### Get metadata
Path: `/:vessel/metadata/:data`  
Method: GET  
Parameters:
* vessel (required): The id of the vessel you want to request
* data (required): A comma seperated list of signals/variables to include in the request

Query:
* from (required): The unix timestamp that is the start of the request range
* to (required): The unix timestamp that is the end of the request range

Returns the metadata laying within the given range (includes metadata only partly within)

### Wipe command
Path: `/command/wipe`  
Method: Post  
Body
* vessel (optional): The id of the vessel you want to request
* data (optional): A comma seperated list of signals/variables to include in the request
* range (optional): A JSON object containing the following:
  * from (optional): The unix timestamp that is the start of the request range
  * to (optional): The unix timestamp that is the end of the request range

Performs delete query on database using the request body as parameters
Be wary that the parameters act as restrictions, meaning no parameters = complete database wipe
To delete everything after a unix timestamp, supply only to, vice versa with from
To delete within a given range, supply both from and to
To only delete from one vessel, supply a vessel in the body
To only delete certain variables, supply a comma-separated list of variables in data

### Stream live data
Path: `/live`  
Method: Upgrade (websocket)  
Query:
* vessel (required): The id of the vessel you want to request
* data (required): A comma seperated list of signals/variables to include in the request

Opens a websocket that automatically fetches second-intervals with a 10 second delay