import { Command, CommandResponse } from '../../proto/wlCommand'
import MqttPool from '../../services/mqtt/mqttPool'
import healthCheckMiddleware from '../../services/cache/healthCheckMiddleware'

/**
 * Performs a wipe command to the cache
 *
 * @param {string|undefined} vessel The vessel to wipe (Optional)
 * @param {string[]|undefined} variables The variables to wipe (Optional)
 * @param {object} ranges The start and end times of the range to wipe (Optional)
 * @returns {Promise<CommandResponse>} The response from the cache
 */
const wipe = async (vessel: string|undefined,
  variables: string[]|undefined,
  ranges: {from: bigint, to: bigint}|undefined): Promise<CommandResponse> => {
  const client = await MqttPool.get()

  try {
    return await healthCheckMiddleware(client, async (resolve, reject) => {
      const command: Command = {
        action: 'wipe',
        data: variables ?? [],
        vessel,
        ranges: ranges ? { from: ranges.from + '', to: ranges.to + '' } : undefined
      }

      client.on('message', (topic, payload) => {
        resolve(CommandResponse.decode(payload))
      })

      client.subscribe('wlcache/command/response/wipe')
      client.publish('wlcache/command/request', Buffer.from(Command.encode(command).finish()))
    })
  } finally {
    MqttPool.release(client)
  }
}
export default wipe
