import healthCheckMiddleware from '../../services/cache/healthCheckMiddleware'
import { MetaDataRequest, MetaDataResponse } from '../../proto/wlMetaData'
import MqttPool from '../../services/mqtt/mqttPool'

/**
 * Retrieves and sends metadata based on the request aprameters
 *
 * @param {string} vessel The vessel to request from
 * @param {number} from The start of the range to request
 * @param {number} to The end of the range to request
 * @param {string[]|undefined} variables The variables to request
 */
const getMetaData = async (vessel: string, from: number, to: number, variables?: string[]|undefined) => {
  const pubTopic = `wlcache/metadata/${vessel}/request`
  const subTopic = `wlcache/metadata/${vessel}/response`

  const client = await MqttPool.get()

  try {
    const response = await healthCheckMiddleware(client, (resolve, reject) => {
      try {
        client.on('message', (topic, payload) => {
          if (topic !== subTopic) return
          const metaResponse = MetaDataResponse.decode(payload)
          if (metaResponse.error != null) {
            return reject(metaResponse.error)
          }

          resolve(metaResponse)
        })
        client.subscribe(subTopic)

        client.publish(pubTopic, Buffer.from(MetaDataRequest.encode({
          variable: variables ?? [],
          from: from + '',
          to: to + ''
        }).finish()))
      } catch (e) {
        reject(e)
      }
    })
    return response
  } finally {
    MqttPool.release(client)
  }
}

export default getMetaData
