import { Changes } from '../../proto/cdplogreader'
import healthCheckMiddleware from '../../services/cache/healthCheckMiddleware'
import MqttPool from '../../services/mqtt/mqttPool'

/**
 * Fetches all the variables availabla from a vessel
 *
 * @param {string} vessel The vessel to discover from
 * @returns {{signals: string[], bytecost: number, bytetransfer: number}} The response from the cache as well as the bytecost and bytetransfer
 */
const discover = async (vessel: string): Promise<{signals: string[], bytecost: number, bytetransfer: number}> => {
  const pubTopic = `wlcache/data/${vessel}/request`
  const subTopic = `wlcache/data/${vessel}/response`

  const client = await MqttPool.get()
  /**
   *
   */
  const execute = async () => {
    return await healthCheckMiddleware<{signals: string[], bytecost: number, bytetransfer: number}>(client, (resolve, reject) => {
      client.subscribe(subTopic)
      client.on('message', (topic, payload) => {
        if (topic !== subTopic) return
        const changes = Changes.decode(payload)
        if (changes.lastN?.nChanges !== '0') return
        const signals = changes.signal
        resolve({
          signals: signals.map(signal => {
            return signal.name
          }),
          bytecost: changes.bytecost!,
          bytetransfer: payload.length!
        })
      })
      client.publish(pubTopic, Buffer.from(Changes.encode({ lastN: { nChanges: '0' }, fromTo: undefined, signal: [] }).finish()))
    })
  }

  try {
    return await execute()
  } finally {
    MqttPool.release(client)
  }
}
export default discover
