import express from 'express'
import requestRange from '../../../../handlers/requests/requestRange'
import { HCFail } from '../../../../services/cache/healthCheckMiddleware'
import requestLastN from '../../../../handlers/requests/requestLastN'

const route = express.Router({ mergeParams: true })

route.get('/', async (req, res) => {
  const { vessel } = (req.params as any)
  if (vessel == null) return res.send('Missing vessel from path')
  if (!/^\w+$/i.test(vessel)) return res.send(`Invalid vessel: "${vessel}"`)

  const data: string[] = (req.params as any).data?.split(',')
  if (data == null) return res.send('Missing data from path')
  if (!Array.isArray(data) && typeof data[0] !== 'string') return res.send('Data must be a string')

  const { lastn, from, to } = req.query
  if (from != null && to != null) {
    try {
      let fromInt = parseInt(from.toString())
      let toInt = parseInt(to.toString())
      if (Number.isNaN(fromInt)) return res.send('From must be a number')
      if (Number.isNaN(toInt)) return res.send('To must be a number')
      fromInt = Math.floor(fromInt)
      toInt = Math.floor(toInt)

      const response = await requestRange(vessel, data, Math.floor(fromInt), Math.floor(toInt))
      if (response?.error != null) {
        res.status(response.error.code)
        return res.send(response.error)
      }
      res.send(response)
    } catch (e: any) {
      if (e?.message === HCFail) {
        res.status(503)
        res.send(HCFail)
      } else {
        res.status(500)
        res.send(e?.message ?? e)
      }
    }
  } else if (lastn != null) {
    try {
      const lastnInt = parseInt(lastn.toString())
      if (!Number.isInteger(lastnInt)) return res.send('Lastn must be a number')

      const response = await requestLastN(vessel, data, lastnInt)

      if (response.error != null) {
        res.status(response.error.code)
        return res.send(response.error)
      }
      res.send(response)
    } catch (e: any) {
      if (e?.message === HCFail) {
        res.status(503)
        res.send(HCFail)
      } else {
        res.status(500)
        res.send(e?.message ?? e)
      }
    }
  } else {
    return res.send('lastn or from and to parameters required')
  }
})

export default route
