/* eslint n/no-callback-literal: "off" */

import mqtt from 'mqtt'
import { HealthCheckSideCar, HealthCheckSideCarObserver } from './healthCheckSideCar'

export const HCFail = 'Cache healthcheck failed'

/**
 * Middleware to ensure that the cache and mqtt broker are both live
 * If either is not responding, the middleware will reject the API request
 *
 * @param {mqtt.MqttClient} client The Mqtt client to use
 * @param {object} callback The resolve/rejects for the function the middleware wraps
 */
const healthCheckMiddleware = async <T>(client: mqtt.MqttClient, callback: (resolve: (t: T) => void, reject: (reason?: any) => void) => void): Promise<T> => {
  let healthCheckObserver: HealthCheckSideCarObserver
  return new Promise<T>(async (resolve, reject) => {
    healthCheckObserver = HealthCheckSideCar.observe(() => {
      reject(new Error('Healthcheck failed'))
    })

    callback((t: T) => {
      resolve(t)
    }, (reason?: any) => {
      console.log('Request error', reason)
      reject(reason)
    })
  }).finally(() => {
    HealthCheckSideCar.cancel(healthCheckObserver)
  })
}
export default healthCheckMiddleware
